require File.join(File.expand_path('../../../helpers', __FILE__),  "document_spec_helper")

def a_document_class
  Doc do
    key :first_name, String
    key :last_name, String
    key :age, Integer
    key :date, Date
  end
end

describe 'a Document' do

  describe 'with an array key' do
    before :each do
      @document = a_document_class
      @document.key :tags, Array
    end

    it 'should give correct default' do
      doc = @document.new
      expect(doc.tags).to eq []
    end

    it 'should work with assignments' do
      doc = @document.new
      doc.tags = %w(foo bar)
      expect(doc.tags).to eq %w(foo bar)
    end

    it 'should work with assignment then <<' do
      doc = @document.new
      doc.tags = []
      doc.tags << "foo"
      expect(doc.tags).to eq ["foo"]
    end

    it 'should work with <<' do
      doc = @document.new
      doc.tags << "foo"
      expect(doc.tags).to eq ["foo"]
    end
  end

  describe 'with an hash key' do
    before :each do
      @document = a_document_class
      @document.key :foo, Hash
    end

    it 'should give correct default' do
      doc = @document.new
      expect(doc.foo).to eq({})
    end

    it 'should work with []=' do
      doc = @document.new
      doc.foo["quux"] = "bar"
      expect(doc.foo["quux"]).to eq "bar"
      expect(doc.foo).to eq({ "quux" => "bar" })
    end

    it 'should work with indifferent access' do
      doc = @document.new
      doc.foo = {:baz => 'bar'}
      expect(doc.foo[:baz]).to eq 'bar'
      expect(doc.foo['baz']).to eq 'bar'
    end
  end

  describe 'custom type key with default' do
    before :each do
      @document = a_document_class
      @document.key :window, WindowSize, :default => WindowSize.new(600, 480)
    end

    it 'should default to default' do
      doc = @document.new
      expect(doc.window).to eq WindowSize.new(600, 480)
    end
  end

  describe 'key with proc default value' do

    before :each do
      @document = a_document_class
      @document.key :proc_default, String, :default => lambda { return 'string' }
    end

    it "should detect and run proc default" do
      doc = @document.new
      expect(doc.proc_default).to eq 'string'
    end
  end

end