require File.join(File.expand_path('../../../helpers', __FILE__),  "document_spec_helper")

describe 'the Document class' do

  before :each do
    @document = Doc()
  end


  it 'should have log method' do
    expect(@document.log).to eq Tomte.logger
    expect(@document.log).to be_instance_of(Tomte::Logger)
  end

end

describe "An instance of a Document" do

  before :each do
    @document = Doc do
      key :name, String
      key :age, Integer
    end
  end

  it 'should create id during initialization' do
    expect(@document.new._id).to be_instance_of(Tomte::ObjectId)
  end

  it 'should have access to log' do
    doc = @document.new
    expect(doc.log).to eq @document.log
    expect(doc.log).to be_instance_of(Tomte::Logger)
  end

  it 'should use default values if defined for keys' do
    @document.key :active, Boolean, :default => true

    expect(@document.new.active).to be_truthy
    expect(@document.new(:active => false).active).to be_falsey
  end

  it 'should use default values if defined even when custom data type' do
    @document.key :window, WindowSize, :default => WindowSize.new(600, 480)

    doc = @document.new
    expect(doc.window).to eq WindowSize.new(600, 480)
  end

  describe "#new" do

    it 'should be true if no id' do
      expect(@document.new.new?).to be_truthy
    end

    it 'should be true if id but using custom id and not saved yet' do
      @document.key :_id, String
      doc = silence_warnings { @document.new }
      doc.id = '1234'
      expect(doc.new?).to be_truthy
    end
  end

  describe 'equality' do
    before :each do
      @oid = Tomte::ObjectId.new
    end

    it 'should delegate hash to _id' do
      doc = @document.new
      expect(doc.hash).to eq doc._id.hash
    end

    it 'should delegate eql to ==' do
      doc = @document.new
      other = @document.new
      expect(doc.eql?(other)).to eq (doc == other)
      expect(doc.eql?(doc)).to eq (doc == doc)
    end

    it 'should know if same object as another' do
      doc = @document.new
      expect(doc).to equal(doc)
      expect(doc).not_to equal(@document.new)
    end

    it 'should be equal if id and class are the same' do
       expect((@document.new('_id' => @oid) == @document.new('_id' => @oid))).to be(true)
    end

    it 'should not be equal if class same but id different' do
       expect((@document.new('_id' => @oid) == @document.new('_id' => Tomte::ObjectId.new))).to be(false)
    end

    it 'should not be equal if id same but class different' do
      another_document = Doc()
      expect((@document.new('_id' => @oid) == another_document.new('_id' => @oid))).to be(false)
    end


  end
end