require File.join(File.expand_path('../../../helpers', __FILE__),  "document_spec_helper")

require 'active_support/time_with_zone'

describe "a tomte Document, regards generic objects" do

  class TestGeneric

    attr_accessor :foo
    attr_accessor :bar

    def initialize(attrs)
      @foo, @bar = [attrs[:foo], attrs[:bar]]
    end

    def self.from_tomte(attrs)
      new(attrs)
    end

    def to_tomte
      {:foo => @foo, :bar => @bar}
    end

  end


  it 'should serialize generics inside hashes' do

    i = TestGeneric.new(:foo => 42, :bar => 16)
    d = {:a => i}

    t = d.to_tomte

    expect(t).to be_a_kind_of Hash
    expect(t[:a]).to be_a_kind_of Hash
    expect(t[:a][:_type]).to eq i.class.name
    expect(Tomte::Document::Parser.is_generic_object?(t[:a])).to be_truthy
    expect(Tomte::Document::Parser.is_document?(t[:a])).to be_falsey
  end


  it 'should serialize generics inside hashes and be reversable' do

    i = TestGeneric.new(:foo => 42, :bar => 'poo')
    d = {:a => i}

    t = d.to_tomte
    expect(Tomte::Document::Parser.is_document?(t)).to be_falsey
    expect(Tomte::Document::Parser.parseable?(t)).to be_truthy
    expect(Tomte::Document::Parser.is_generic_object?(t[:a])).to be_truthy

    r = Tomte::Document::Parser.load(t)
    expect(r).to be_a_kind_of Hash
    expect(r[:a]).to be_a_kind_of TestGeneric
    expect(r[:a].foo).to eq 42
    expect(r[:a].bar).to eq 'poo'

  end
end