require File.join(File.expand_path('../../../helpers', __FILE__),  "document_spec_helper")

class MyCollectorDoc
  include Tomte::Document

  key :foo, Tomte::OpenDocument, :default => lambda { Tomte::OpenDocument.new }

end

class MyCollectorDocCast
  include Tomte::Document

  key :foo, Tomte::OpenDocument, :default => lambda { Tomte::OpenDocument.new },
      :cast => true

end

describe 'an OpenDocument inside a doc' do

  it 'should properly have a default' do
    d = MyCollectorDoc.new
    expect(d.foo).to be_a_kind_of Tomte::OpenDocument

  end

  it 'should able to take the OpenDocument at init' do
    d = MyCollectorDoc.new(:foo =>
      Tomte::OpenDocument.new(:a => 1, :b => 2)
    )
    expect(d.foo).to be_a_kind_of Tomte::OpenDocument

    expect(d.foo.a).to eq 1
    expect(d.foo.b).to eq 2

    expect(d.foo.table[:_id]).to be_nil
    expect(d.foo.table[:_doctype]).to be_nil
    expect(d.foo.table['_id']).to be_nil
    expect(d.foo.table['_doctype']).to be_nil
  end

  it 'should able to take an assigned OpenDocument' do
    d = MyCollectorDoc.new
    d.foo = Tomte::OpenDocument.new(:a => 1, :b => 2)

    expect(d.foo).to be_a_kind_of Tomte::OpenDocument

    expect(d.foo.a).to eq 1
    expect(d.foo.b).to eq 2

    expect(d.foo.table[:_id]).to be_nil
    expect(d.foo.table[:_doctype]).to be_nil
    expect(d.foo.table['_id']).to be_nil
    expect(d.foo.table['_doctype']).to be_nil
  end

end

describe 'an OpenDocument inside a doc with cast option' do

  it 'should able to take the OpenDocument at init' do
    d = MyCollectorDoc.new(:foo =>
      Tomte::OpenDocument.new(:a => 1, :b => 2)
    )
    expect(d.foo).to be_a_kind_of Tomte::OpenDocument

    expect(d.foo.a).to eq 1
    expect(d.foo.b).to eq 2

    expect(d.foo.table[:_id]).to be_nil
    expect(d.foo.table[:_doctype]).to be_nil
    expect(d.foo.table['_id']).to be_nil
    expect(d.foo.table['_doctype']).to be_nil
  end

  it 'should able to take an assigned OpenDocument' do
    d = MyCollectorDoc.new
    d.foo = Tomte::OpenDocument.new(:a => 1, :b => 2)

    expect(d.foo).to be_a_kind_of Tomte::OpenDocument

    expect(d.foo.a).to eq 1
    expect(d.foo.b).to eq 2

    expect(d.foo.table[:_id]).to be_nil
    expect(d.foo.table[:_doctype]).to be_nil
    expect(d.foo.table['_id']).to be_nil
    expect(d.foo.table['_doctype']).to be_nil
  end

  it 'should able to take the hash at init' do
    d = MyCollectorDoc.new(:foo =>
      {:a => 1, :b => 2}
    )

    expect(d.foo).to be_a_kind_of Tomte::OpenDocument

    expect(d.foo.a).to eq 1
    expect(d.foo.b).to eq 2

    expect(d.foo.table[:a]).to eq 1
    expect(d.foo.table[:b]).to eq 2

    expect(d.foo.table[:_id]).to be_nil
    expect(d.foo.table[:_doctype]).to be_nil
    expect(d.foo.table['_id']).to be_nil
    expect(d.foo.table['_doctype']).to be_nil
  end
end
